/**
  ******************************************************************************
  * @file    stm32xx_lppuart_interfaces.h
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   Header for stm32xx_lpp_uart_interfaces.c module
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __STM32XX_LPPUART_INTERFACES_H
#define __STM32XX_LPPUART_INTERFACES_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32xx_lppuart_conf.h"

/* Exported types ------------------------------------------------------------*/

 /**
   * The UART driver manages the Rx and Tx path separately
   * These are the identifier the UART driver uses to request or release the HS clock
   * of the related path
   */
 typedef enum{
	 eLPPUART_Tx,							/**<  The UART transmitter */
	 eLPPUART_Rx							/**<  The UART receiver */
}eLPPUART_HSclkRequester_t;

/**
  * These are the requests from the UART driver to the low power manager regarding the availability of the HS clock
  */
typedef enum{
	 eLPPUART_HSclkDisable,				/**<  The HS clock may be disabled */
	 eLPPUART_HSclkEnable					/**<  The HS clock shall be kept enabled */
}eLPPUART_HSclkMode_t;

/**
  *	These are the notification regarding transfer with the BLE connectivity device
  *	The source is the STM32 and the destination is the BLE connectivity device
  */
  typedef enum{
	 eLPPUART_TxDone,						/**<  The transmission has been completed */
	 eLPPUART_RxDone						/**<  A packet has been received */
 }eLPPUART_PhyDriverNotification_t;

 /**
   * Informations to be reported to the BLE module in the UART interrupt handler
   */
 typedef struct{
	 eLPPUART_PhyDriverNotification_t	ePhyDriverNotification;	/**<  Event to be reported to the BLE module */
 	 uint32_t							uwParam1;				/**<  Reserved for future use */
 	 uint32_t							uwParam2;				/**<  Reserved for future use */
 }sLPPUART_PhyDriverEvent_t;

 /**
   * Type of the UART interrupt handler
   */
 typedef void (*pf_LPPUART_PhyDriverEvent_Handler_t)(sLPPUART_PhyDriverEvent_t pLPPUART_PhyDriverEvent);

/* Exported defines --------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/
/* Exported macros -----------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */ 
 void LPPUART_uart_open(pf_LPPUART_PhyDriverEvent_Handler_t pf_PhyDriverEvent_Handler);
 void LPPUART_send_data(uint8_t * pbuffer, uint16_t uhlength);
 void LPPUART_receive_data(uint8_t * pbuffer);
 void LPPUART_HSclkRequest(eLPPUART_HSclkRequester_t eHSclkRequester, eLPPUART_HSclkMode_t eHSclkMode);
 /**
   * @brief	This is the CTS interrupt handler required for the Low Power Protocol implemented over UART
   * @note  It shall be called in the EXTI15_10_IRQHandler by the application ONLY when either USART1 or USART3 is selected
   * 		When USART2 is selected, the UART driver is taking care of it.
   *
   * @param  None
   * @note   None
   * @retval None
   */
 void LPPUART_wcts_handler(void);

 /**
   * @brief	This is UART interrupt handler required for the Low Power Protocol implemented over UART
   *
   * @param  None
   * @note   None
   * @retval None
   */
 void LPPUART_uart_handler(void);

 /**
   * @brief	This is DMA interrupt handler required for the Low Power Protocol implemented over UART
   *
   * @param  None
   * @note   None
   * @retval None
   */
 void LPPUART_dma_rx_handler(void);

 /**
   * @brief	This is the handler called by the driver when a full message is received from the nRF51 device
   * @note  The purpose of this function is to handle the received message out of the UART interrupt context.
   * 		The application may either:
   * 			a) send a signal to a process in case an OS is implemented
   * 			b) set a flag to be polled in the idle context
   * 			c) keep running in the UART context
   * 		The event handler registered by the BLE module shall be called in the new context.
   *
   * @param1 The event handler registered by the BLE module to process the message received from the nRF51
   * @note   None
   * @param2 The parameter to be passed to the event handler registered by the BLE module (param1)
   * @note   None
   * @retval None
   */
 void LPPUART_Msg_Handler(pf_LPPUART_PhyDriverEvent_Handler_t pf_PhyDriverEvent_Handler, sLPPUART_PhyDriverEvent_t pPhyDriverEvent);


#ifdef __cplusplus
}
#endif

#endif /*__STM32XX_LPPUART_INTERFACES_H */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
