/**
  ******************************************************************************
  * @file    stm32xx_lppuart.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   UART Driver Low Power Protocol
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2014 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "stm32xx_lppuart_conf.h"

/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/

/**
 *	List of modules that shall grant the notification to be sent to the application
 *
 *	The UART driver decodes on its own the header of the received packet
 *	Even though the application did not yet request to receive the payload, the UART driver
 *	starts reception
 *	The payload receive notification cannot be sent until all these modules gave acceptance
 *
 *	It supports only 8 members. To support more members, the type of ubRxPayloadRequest shall be changed
 *	When a new member is added, the definition of ENABLE_PAYLOAD_NOTIFICATION shall be updated
 */
typedef enum{
	eApplication,		/**<  The application has requested to receive the payload */
	eBLEModule			/**<  The payload has been fully received and is ready to be sent to the application */
}eRxPayloadNotificationRequester_t;

/**
 *	States to defined which part of the packet the Host is waiting for
 *
 *	The UART driver decodes on its own the header of the received packet
 *	Even though the application did not yet request to receive the payload, the UART driver
 *	starts reception
 *	The first receive request from the application shall be the Header. The next receive request from the application
 *	shall be the payload
 *	These two states track which part of the packet the Host is expecting.
 */
typedef enum{
	eHostHeaderToRx,	/**<  The application is expecting to receive the header */
	eHostPayloadToRx	/**<  The application is expecting to receive the payload */
}eHostRxState_t;

/**
 *	States to defined which part of the packet is being received by the UART driver
 *
 * The UART low power protocol requires specific actions on different steps while receiving a packet
 * This is the list of steps where dedicated actions shall be taken
 */
typedef enum{
	eHeaderToRx,		/**<  The UART driver is receiving the header */
	ePayloadToRx,		/**<  The UART driver is receiving the payload */
	eLast2ndByteToRx,	/**<  The application is receiving the last second byte of the payload (n-1) */
	eLastByteToRx		/**<  The application is receiving the last byte of the payload */
}eRxState_t;

/**
 *	List of event being tracked by the UART driver
 *
 * The UART low power protocol requires some acknowledge on different steps while transmitting a packet
 * This is the list of steps where dedicated actions shall be taken.
 * There are all based on CTS interrupt at different time of the UART activity
 */
typedef enum{
	eRemoteTxrequest,		/**<  The UART driver is listening for a Remote transmit request */
	eLastSentByteAck,		/**<  The UART driver is waiting for the ack of the last byte sent  of the payload */
	eLast2ndSentByteAck,	/**<  The UART driver is waiting for the ack of the last second byte sent (n-1) of the payload */
	eAckToTxRequest			/**<  The UART driver is waiting for the ack of a transmit request */
}eWCTSEventWaited_t;

/**
 *	Event being tracked by the UART Interrupt Handler when sending a packet
 *
 * The UART low power protocol requires dedicated actions when sending the last two byte of the packet
 */
typedef enum{
	ePayloadSent,	/**<  The second last byte (n-1) has been shifted out from the UART */
	eLastByteSent	/**<  The last byte has been shifted out from the UART */
}eUSART_TC_EventWaited_t;

/**
 *	Context of the UART driver
 *
 * The UART Low power driver stores several informations to handle properly the reception and transmission of a message
 * These are all stored in this structure instead of being scattered across several variables
 */
typedef struct{
	pf_LPPUART_PhyDriverEvent_Handler_t	pf_PhyDriverEvent_Handler;	/**<  BLE Event handler to be called to notify end of either RX or TX */
	uint32_t								uwTxFlag;					/**<  This flag is tracking the transmit activity to handle low power protocol */
	uint32_t								uwRxFlag;					/**<  This flag is tracking the receive activity  to handle low power protocol */
	eWCTSEventWaited_t						eWCTSEventWaited;			/**<  The type of event expected on the CTS interrupt  */
	eUSART_TC_EventWaited_t					eUSART_TC_EventWaited;		/**<  The part of the packet being shifted out the UART  */
	eRxState_t								eRxState;					/**<  The UART driver Rx state machine to apply the low power protocol */
	eHostRxState_t							eHostRxState;				/**<  The part of the packet the BLE application is expected */
	uint32_t								uwWCTS_TxStarted;			/**<  Used in the CTS interrupt handler to know when transmit is ongoing */
	uint16_t								uhTxSize;					/**<  The size from the header of the transmit packet */
	uint16_t								uhRxSize;					/**<  The size from the header of the receive packet */
	uint8_t*								pTxBuffer;					/**<  The address of the transmit buffer */
	uint8_t*								pRxBuffer;					/**<  The address of the receive buffer */
}sUartContext_t;


/* Private macros ------------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/

/**
 *	Packet Header size
 *
 *	It defines the packet header size (in bytes) that the UART driver is expected at the start of a message
 */
#define	PACKET_HEADER_SIZE			((uint32_t)2)

/**
 *	Condition to notify the payload reception to the BLE module
 *
 *	To enable the payload notification, all modules shall set their bit
 */
#define	ENABLE_PAYLOAD_NOTIFICATION		0x03


/* Private variables ---------------------------------------------------------*/

/**
 *	UART context
 *
 *	Variable to hold the UART context to be used in the UART driver
 */
static volatile sUartContext_t sUartContext;

/**
 *	Flag to lock the Rx payload notification to the BLE module
 *
 *	This is a container of all modules that shall allow the Rx payload notification to the BLE module
 */
static volatile uint8_t	ubRxPayloadRequest;

/**
 *	DMA Handle
 */
static DMA_HandleTypeDef hdma_tx;		/**< DMA handle of the TX path */
static DMA_HandleTypeDef hdma_rx;		/**< DMA handle of the RX path */
static UART_HandleTypeDef huart;		/**< UART handle */


/* Private function prototypes -----------------------------------------------*/
static void uart_init(void);
static void gpio_rx_cts_init(void);
static void gpio_tx_rts_init(void);
static void dma_init(void);
static void wcts_interrupt_enable(eWCTSEventWaited_t eWCTSEventWaited);
static void wcts_interrupt_disable(void);
static void set_wrts_gpio_mode(uint32_t eGPIO_mode);
static void RxPayloadNotification(eRxPayloadNotificationRequester_t eRxPayloadNotificationRequester);
static void SetGPIO_PUPD(GPIO_TypeDef* GPIOx, uint32_t GPIO_Pin_Source, uint32_t GPIOPuPd);


/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Set the GPIO PU/PD configuration
  *
  * @note   This API shall never be called in a PRIMASK critical section as it would re-enable
  * 		the PRIMASK bit before this would be expected by the calling function
  *
  * @param 	GPIOx : The GPIO port on which the PU/PD configuration shall apply
  *
  * @param 	GPIO_Pin_Source : The GPIO pin on which the PU/PD configuration shall apply
  *
  * @param 	GPIOPuPd : The PU/PD configuration that shall be applied
  *
  * @retval None
  */
static void SetGPIO_PUPD(GPIO_TypeDef* GPIOx, uint32_t GPIO_Pin_Source, uint32_t GPIOPuPd)
{
	/**
	 * The access to the GPIO register may be colliding with an access from the application
	 * The operation is protected by a critical section to avoid any corruption
	 */
	LPPUART_ENTER_CRITICAL_SECTION;
	HAL_LPPUART_GPIO_Set_PUPD(GPIOx, GPIO_Pin_Source, GPIOPuPd);
	LPPUART_EXIT_CRITICAL_SECTION;

	return;
}

/**
  * @brief  Rx Payload Application Notification
  *
  * @note	The payload notification cannot be sent before the BLE application module
  * 		has requested to receive the payload
  * 		The UART driver is receiving the payload on its own based on the length
  * 		decoded in the header. It may happen the UART driver has received the payload before
  * 		the BLE uart module has made the request
  * 		The payload notification shall be notified to the BLE application when both :
  * 		a) The BLE application made the request to receive the payload
  * 		b) The UART driver has received the payload
  *
  * 		This API shall never be called in a PRIMASK critical section as it would re-enable
  * 		the PRIMASK bit before this would be expected by the calling function.
  *
  * @param  eRxPayloadNotificationRequester : The module that is ready to notify the payload to the BLE application
  *
  * @retval None
  */
static void RxPayloadNotification(eRxPayloadNotificationRequester_t eRxPayloadNotificationRequester)
{
	sLPPUART_PhyDriverEvent_t sPhyDriverEvent;

	/**
	 * The container may be access from two different contexts
	 * The operation is protected by a critical section to avoid any corruption
	 */
	LPPUART_ENTER_CRITICAL_SECTION;
	ubRxPayloadRequest |= (1<<eRxPayloadNotificationRequester);
	LPPUART_EXIT_CRITICAL_SECTION;

	if(ubRxPayloadRequest == ENABLE_PAYLOAD_NOTIFICATION)
	{
		/* notify payload reception */
		ubRxPayloadRequest = 0;

		sPhyDriverEvent.ePhyDriverNotification = eLPPUART_RxDone;
		sPhyDriverEvent.uwParam1 = (uint32_t)sUartContext.pRxBuffer;
		LPPUART_Msg_Handler(sUartContext.pf_PhyDriverEvent_Handler, sPhyDriverEvent);
	}

	return;
}

/**
  * @brief  Set the RTS GPIO Mode
  *
  * @note	This API shall never be called in a PRIMASK critical section as it would re-enable
  * 		the PRIMASK bit before this would be expected by the calling function.
  *
  * @param  eGPIO_mode : The mode to be used with the RTS GPIO pin
  *
  * @retval None
  */
static void set_wrts_gpio_mode(uint32_t eGPIO_mode)
{
	/**
	 * The access to the GPIO register may be colliding with an access from the application
	 * The operation is protected by a critical section to avoid any corruption
	 */
	LPPUART_ENTER_CRITICAL_SECTION;
	HAL_LPPUART_GPIO_Set_Mode(USART_WRTS_GPIO_PORT, USART_WRTS_PIN_SOURCE, eGPIO_mode);
	LPPUART_EXIT_CRITICAL_SECTION;

	return;
}


/**
  * @brief  Disable CTS interrupt
  *
  * @note	The CTS interrupt is always kept enable in the NVIC. It is disable only in the EXTI block
  *
  * 		This API shall never be called in a PRIMASK critical section as it would re-enable
  * 		the PRIMASK bit before this would be expected by the calling function.
  *
  * @param  None
  *
  * @retval None
  */
static void wcts_interrupt_disable(void)
{
	/**
	 * The access to the EXTI register may be colliding with an access from the application
	 * The operation is protected by a critical section to avoid any corruption
	 */
	LPPUART_ENTER_CRITICAL_SECTION;
	__HAL_LPPUART_EXTI_DISABLE_IT(USART_WCTS_PIN);	/**< Mask out CTS interrupt in EXTI block */
	LPPUART_EXIT_CRITICAL_SECTION;

	/**
	 * As the register is only written, the operation cannot collide with other access
	 * The operation is out of critical section
	 */
	__HAL_GPIO_EXTI_CLEAR_IT(USART_WCTS_PIN);	/**< Clear pending bit in EXTI block */

	return;
}

/**
  * @brief  Enable CTS interrupt
  *
  * @note	The UART low power protocol is using the CTS line to request transmission
  * 		and to ack packet reception (in combination with HW UART flow control)
  * 		When handling the CTS line as low power control, the actions are taken
  * 		based on the level of the line
  * 		The STM32 supports only edge detection with the EXTI block to generate an interrupt
  * 		To limit the number of access dynamically to the EXTI block, the line is configured
  * 		to generate an interrupt on both rising and falling edge (that means on any change of the IO level)
  * 		When the CTS interrupt is enable, it is passed as parameter which event the FW is waiting for
  * 		Based on the expected event, the FW knows what is the expected level.
  * 		In order to not miss the changing level when the CTS pulse is very short, the FW is:
  * 		a) First enabling CTS changing state detection
  * 		b) Checking the IO level in case the pulse
  *
  *
  * 		This API shall never be called in a PRIMASK critical section as it would re-enable
  * 		the PRIMASK bit before this would be expected by the calling function.
  *
  * @param  eWCTSEventWaited
  *
  * @retval None
  */
static void wcts_interrupt_enable(eWCTSEventWaited_t eWCTSEventWaited)
{
	/**
	 *  Start of critical section
	 */
	LPPUART_ENTER_CRITICAL_SECTION;

	if((eWCTSEventWaited != eRemoteTxrequest) ||(sUartContext.uwWCTS_TxStarted == DISABLE))
	{
		sUartContext.eWCTSEventWaited = eWCTSEventWaited;

		/**
		 *  Enable WCTS interrupt in EXTI block
		 */
		__HAL_LPPUART_EXTI_ENABLE_IT(USART_WCTS_PIN);

		switch(eWCTSEventWaited)
		{
			case eLastSentByteAck:
				/**
				 * Check WCTS GPIO line
				 */
				if(HAL_GPIO_ReadPin(USART_WCTS_GPIO_PORT, USART_WCTS_PIN) == GPIO_PIN_SET)
				{
					/**
					 * Edge has been missed - Set the bit manually
					 */
					__HAL_GPIO_EXTI_GENERATE_SWIT(USART_WCTS_PIN);
				}
				break;

			case eLast2ndSentByteAck:
			case eAckToTxRequest:
			case eRemoteTxrequest:
				/**
				 * Check WCTS GPIO line
				 */
				if(HAL_GPIO_ReadPin(USART_WCTS_GPIO_PORT, USART_WCTS_PIN) == GPIO_PIN_RESET)
				{
					/**
					 * Edge has been missed - Set the bit manually
					 */
					__HAL_GPIO_EXTI_GENERATE_SWIT(USART_WCTS_PIN);
				}
				break;

			default:
				break;
		}

	}

	/**
	 * end of critical section
	 */
	LPPUART_EXIT_CRITICAL_SECTION;

	return;
}

/**
  * @brief  Initialize DMA
  *
  * @note	This API sets the NVIC and the static configuration of the DMA block
  *
  * @param  None
  *
  * @retval None
  */
static void dma_init(void)
{
	uint32_t uart_data_register_address;
	/**
	 * DMA configuration for each channel
	 */
	LPPUART_msp_dma_set_config(&hdma_tx, &hdma_rx);

	/**
	 * Configure UART2 TX path
	 */
	uart_data_register_address = __HAL_LPPUART_UART_GET_TX_DATA_REGISTER_ADDRESS(&huart); /**<Get the Tx address of the UART Data register */
	hdma_tx.Instance = DMA_USART_TX_CHANNEL;
	hdma_tx.State = HAL_DMA_STATE_READY;
	hdma_tx.Lock = HAL_UNLOCKED;
	HAL_DMA_Init(&hdma_tx);
	__HAL_LPPUART_DMA_SET_PERIPHERAL_ADDRESS(&hdma_tx, uart_data_register_address);

	/**
	 * Configure UART2 RX path
	 */
	uart_data_register_address = __HAL_LPPUART_UART_GET_RX_DATA_REGISTER_ADDRESS(&huart); /**<Get the Rx address of the UART Data register */
	hdma_rx.Instance = DMA_USART_RX_CHANNEL;
	hdma_rx.State = HAL_DMA_STATE_READY;
	hdma_rx.Lock = HAL_UNLOCKED;
	HAL_DMA_Init(&hdma_rx);
	__HAL_LPPUART_DMA_SET_PERIPHERAL_ADDRESS(&hdma_rx, uart_data_register_address);
	__HAL_DMA_ENABLE_IT(&hdma_rx, DMA_IT_TC);

	/**
	 * Set NVIC priority
	 */
	HAL_NVIC_SetPriority(DMA_USART_RX_NVIC_VECTOR, NVIC_UART_DMA_IT_PREEMPTPRIORITY, NVIC_UART_DMA_IT_SUBPRIORITY);

	/**
	 * Enable NVIC interrupt
	 */
	HAL_NVIC_EnableIRQ(DMA_USART_RX_NVIC_VECTOR);

    return;
}

/**
  * @brief  Initialize UART
  *
  * @note	This API sets the NVIC and the static configuration of the UART block
  *
  * @param  None
  *
  * @retval None
  */
static void uart_init(void)
{
	/*
	 * Initialize the handle
	 */
	huart.Instance = USART_BLE;
	huart.Lock = HAL_UNLOCKED;
	huart.State = HAL_UART_STATE_READY;

	/**
	 * Enable UART block
	 */
	__HAL_UART_ENABLE(&huart);

    /**
     * Clear TC Flag
     */
   	__HAL_UART_CLEAR_IT(&huart, UART_CLEAR_TCF);

    /**
    *  USART configuration
    * Word Length = 8 Bits
    * One Stop Bit
    * No parity
    * CTS/RTS Hardware flow control enabled.
    * Receive and transmit enabled
    * OverSample = 8
    */
   	huart.Init.BaudRate = USART_BAUDRATE;
   	huart.Init.WordLength = UART_WORDLENGTH_8B;
   	huart.Init.StopBits = UART_STOPBITS_1;
   	huart.Init.Parity = UART_PARITY_NONE;
   	huart.Init.Mode = UART_MODE_TX_RX;
   	huart.Init.HwFlowCtl = UART_HWCONTROL_RTS_CTS;
   	huart.Init.OverSampling = UART_OVERSAMPLING_8;
   	huart.Init.OneBitSampling = UART_ONEBIT_SAMPLING_DISABLED;
   	HAL_UART_Init(&huart);

    /**
     * Wait IDLE character to be sent
     */
    while(__HAL_UART_GET_IT(&huart, UART_IT_TC) == RESET);

    /**
     * Clear TC Flag
     */
   	__HAL_UART_CLEAR_IT(&huart, UART_CLEAR_TCF);

	/**
	 * Enable TC interrupt
	 */
	__HAL_UART_ENABLE_IT(&huart, UART_IT_TC);

	/**
	 * Set NVIC priority
	 */
	HAL_NVIC_SetPriority(USART_NVIC_VECTOR, NVIC_UART_UART_IT_PREEMPTPRIORITY, NVIC_UART_UART_IT_SUBPRIORITY);

	/**
	 * Enable USART interrupt in NVIC
	 */
	HAL_NVIC_EnableIRQ(USART_NVIC_VECTOR);

    /**
	 * enable DMA in UART2 for receive and transmit
	 */
	__HAL_LPPUART_UART_ENABLE_DMAREQ(&huart, USART_CR3_DMAR | USART_CR3_DMAT);

	return;
}

/**
  * @brief  Initialize GPIO for the UART INPUT lines
  *
  * @note   All registers from a port used by the BLE UART shall not be changed by the application when
  * 		this API is running
  *
  * @param  None
  *
  * @retval None
  */
static void gpio_rx_cts_init(void)
{
	GPIO_InitTypeDef sGPIO_InitStruct;

	/**
	 * UART RX Pin
	 */
	sGPIO_InitStruct.Pin = USART_RX_PIN;
	sGPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	sGPIO_InitStruct.Pull = GPIO_PULLUP;
	sGPIO_InitStruct.Speed = GPIO_SPEED_FAST;
	sGPIO_InitStruct.Alternate = USART_RX_AF;
	HAL_GPIO_Init(USART_RX_GPIO_PORT, &sGPIO_InitStruct);


	/**
	 * UART CTS Pin
	 */
	sGPIO_InitStruct.Pin = USART_WCTS_PIN;
	sGPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	sGPIO_InitStruct.Pull = GPIO_PULLUP;
	sGPIO_InitStruct.Speed = GPIO_SPEED_HIGH;
	sGPIO_InitStruct.Alternate = USART_WCTS_AF;
	HAL_GPIO_Init(USART_WCTS_GPIO_PORT, &sGPIO_InitStruct);

	/**
	 * Select WCTS on EXTI
	 */
	HAL_LPPUART_SYSCFG_Set_EXTI_Conf(USART_WCTS_SYSCFG_EXTI_PORT, USART_WCTS_PIN_SOURCE);

	/**
	 * Enable both edge
	 */
	__HAL_LPPUART_EXTI_ENABLE_RISING_EDGE(USART_WCTS_PIN);
	__HAL_LPPUART_EXTI_ENABLE_FALLING_EDGE(USART_WCTS_PIN);

	/**
	 * Set NVIC priority
	 */
	HAL_NVIC_SetPriority(USART_WCTS_NVIC_VECTOR, NVIC_UART_WCTS_IT_PREEMPTPRIORITY, NVIC_UART_WCTS_IT_SUBPRIORITY);

	/**
	 *	Enable NVIC interrupt
	 */
	HAL_NVIC_EnableIRQ(USART_WCTS_NVIC_VECTOR);

    return;
}

/**
  * @brief  Initialize GPIO for the UART OUTPUT lines
  *
  * @note   All registers from a port used by the BLE UART shall not be changed by the application when
  * 		this API is running
  *
  * @param  None
  *
  * @retval None
  */
static void gpio_tx_rts_init(void)
{
	GPIO_InitTypeDef sGPIO_InitStruct;

	/**
	 * UART TX Pin
	 */
	sGPIO_InitStruct.Pin = USART_TX_PIN;
	sGPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	sGPIO_InitStruct.Pull = GPIO_NOPULL;
	sGPIO_InitStruct.Speed = GPIO_SPEED_FAST;
	sGPIO_InitStruct.Alternate = USART_TX_AF;
	HAL_GPIO_Init(USART_TX_GPIO_PORT, &sGPIO_InitStruct);

	/**
	 * UART RTS Pin
	 */
	HAL_GPIO_WritePin(USART_WRTS_GPIO_PORT, USART_WRTS_PIN, GPIO_PIN_SET);

	sGPIO_InitStruct.Pin = USART_WRTS_PIN;
	sGPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	sGPIO_InitStruct.Pull = GPIO_NOPULL;
	sGPIO_InitStruct.Speed = GPIO_SPEED_FAST;
	sGPIO_InitStruct.Alternate = USART_WRTS_AF;		/**< As the IO is configured as Output, this parameter is not considered */
	HAL_GPIO_Init(USART_WRTS_GPIO_PORT, &sGPIO_InitStruct);
	HAL_LPPUART_GPIO_Set_PinAFConfig(USART_WRTS_GPIO_PORT,USART_WRTS_PIN_SOURCE,USART_WRTS_AF);	/**< Alternate mode is now ready */

    return;
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief  DMA Rx Handler
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void LPPUART_dma_rx_handler(void)
{
	sLPPUART_PhyDriverEvent_t sPhyDriverEvent;

	sUartContext.uwRxFlag = ENABLE;

	__HAL_DMA_CLEAR_FLAG(&hdma_rx, DMA_USART_RX_CHANNEL_FLAG);

	sUartContext.uhRxSize = sUartContext.pRxBuffer[0] + (sUartContext.pRxBuffer[1] << 8);

	switch(sUartContext.eRxState)
	{
		case eHeaderToRx:
			/**
			 * The DMA could be used only to receive a payload with more that two bytes
			 */
			if(sUartContext.uhRxSize > 2)
			{
				/**
				 * Go with DMA to receive other data
				 */
				sUartContext.eRxState = ePayloadToRx;
				__HAL_DMA_DISABLE(&hdma_rx);
				__HAL_LPPUART_DMA_SET_COUNTER(&hdma_rx, sUartContext.uhRxSize-PACKET_HEADER_SIZE);
				__HAL_LPPUART_DMA_SET_MEMORY_ADDRESS(&hdma_rx, (uint32_t)(sUartContext.pRxBuffer+PACKET_HEADER_SIZE));
				__HAL_DMA_ENABLE(&hdma_rx);

				sPhyDriverEvent.ePhyDriverNotification = eLPPUART_RxDone;
				sPhyDriverEvent.uwParam1 = (uint32_t)sUartContext.pRxBuffer;
				LPPUART_Msg_Handler(sUartContext.pf_PhyDriverEvent_Handler, sPhyDriverEvent);
			}
			else
			{
				/**
				 * Go with USART interrupt to receive end of packet
				 */
				sUartContext.eRxState = eLast2ndByteToRx;
				/**
				 * Enable RX interrupt
				 */
				__HAL_UART_ENABLE_IT(&huart, UART_IT_RXNE);
			}

			LPPUART_HSclkRequest(eLPPUART_Rx, eLPPUART_HSclkEnable);

			break;

		case ePayloadToRx:
			/**
			 * Go with USART interrupt to receive end of packet
			 */
			sUartContext.eRxState = eLast2ndByteToRx;
			/**
			 * Enable RX interrupt
			 */
			__HAL_UART_ENABLE_IT(&huart, UART_IT_RXNE);
			break;

		default:
			break;
	}

	return;
}

/**
  * @brief  UART BLE Handler
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void LPPUART_uart_handler(void)
{
	sLPPUART_PhyDriverEvent_t sPhyDriverEvent;
	uint16_t	localRxSize;
	uint8_t		*plocalRxbuffer;

	if( (__HAL_UART_GET_IT(&huart, UART_IT_TC) != RESET) && (__HAL_UART_GET_IT_SOURCE(&huart, UART_IT_TC) != RESET) )
	{
		/**
		 * Clear TC Flag
		 */
	   	__HAL_UART_CLEAR_IT(&huart, UART_CLEAR_TCF);

		if(sUartContext.eUSART_TC_EventWaited == ePayloadSent)
		{
			wcts_interrupt_enable(eLast2ndSentByteAck);
			sUartContext.eUSART_TC_EventWaited = eLastByteSent;
		}
		else
		{
			sUartContext.eUSART_TC_EventWaited = ePayloadSent;

			LPPUART_HSclkRequest(eLPPUART_Tx, eLPPUART_HSclkDisable);

			/**
			 * Notify end of transmit
			 */
			sPhyDriverEvent.ePhyDriverNotification = eLPPUART_TxDone;
			LPPUART_Msg_Handler(sUartContext.pf_PhyDriverEvent_Handler, sPhyDriverEvent);
		}
	}

	if( (__HAL_UART_GET_IT(&huart, UART_IT_RXNE) != RESET) && (__HAL_UART_GET_IT_SOURCE(&huart, UART_IT_RXNE) != RESET) )
	{
		switch(sUartContext.eRxState)
		{
			case eLast2ndByteToRx:
				SetGPIO_PUPD(USART_WCTS_GPIO_PORT, USART_WCTS_PIN_SOURCE, GPIO_PULLUP);
				plocalRxbuffer = sUartContext.pRxBuffer;
				localRxSize = sUartContext.uhRxSize;
				*(plocalRxbuffer + localRxSize) = *(uint8_t*)__HAL_LPPUART_UART_GET_RX_DATA_REGISTER_ADDRESS(&huart);

				/**
				 * Go with USART interrupt to receive end of packet
				 */
				sUartContext.eRxState = eLastByteToRx;
				break;

			case eLastByteToRx:
				/**
				 * Stop flow
				 */
				HAL_GPIO_WritePin(USART_WRTS_GPIO_PORT, USART_WRTS_PIN, GPIO_PIN_SET);
				set_wrts_gpio_mode(GPIO_MODE_OUTPUT_PP);

				/**
				 * Disable the DMAR_Request
				 */
				__HAL_LPPUART_UART_DISABLE_DMAREQ(&huart,  USART_CR3_DMAR);

				/**
				 * read byte
				 */
				plocalRxbuffer = sUartContext.pRxBuffer;
				localRxSize = sUartContext.uhRxSize;
				*(plocalRxbuffer + localRxSize + 1) = *(uint8_t*)__HAL_LPPUART_UART_GET_RX_DATA_REGISTER_ADDRESS(&huart);
				sUartContext.uwRxFlag = DISABLE;
				/**
				 * Disable RX interrupt
				 */
				__HAL_UART_DISABLE_IT(&huart, UART_IT_RXNE);
				if(sUartContext.uwTxFlag == ENABLE)
				{
					/**
					 * Enable receiving path
					 */
					set_wrts_gpio_mode(GPIO_MODE_AF_PP);
				}
				else
				{
					/**
					 * Enable WCTS interrupt
					 */
					wcts_interrupt_enable(eRemoteTxrequest);
				}

				sUartContext.eRxState = eHeaderToRx;
				/**
				 * Enable the DMAR_Request
				 */
				__HAL_LPPUART_UART_ENABLE_DMAREQ(&huart, USART_CR3_DMAR);

				LPPUART_HSclkRequest(eLPPUART_Rx, eLPPUART_HSclkDisable);

				RxPayloadNotification(eBLEModule);
				break;

			default:
				break;
		}
	}

	return;
}

/**
  * @brief  Allocate all resources to get the UART ready to be used
  *
  * @note   The UART implementation is using a DMA and supporting the Low Power Control Mechanism
  *
  * @param  Param1     The handler to be called when data are received over UART
  *
  * @retval None
  */
void LPPUART_uart_open(pf_LPPUART_PhyDriverEvent_Handler_t pf_PhyDriverEvent_Handler)
{
	/**
	 * UART context configuration
	 */
	sUartContext.pf_PhyDriverEvent_Handler = pf_PhyDriverEvent_Handler;
	sUartContext.uwRxFlag = DISABLE;
	sUartContext.uwTxFlag = DISABLE;
	sUartContext.eRxState = eHeaderToRx;
	sUartContext.eWCTSEventWaited = eRemoteTxrequest;
	sUartContext.eHostRxState = eHostHeaderToRx;
	sUartContext.uwWCTS_TxStarted = DISABLE;
	sUartContext.eUSART_TC_EventWaited = ePayloadSent;

	ubRxPayloadRequest = 0;

	/**
	 * Configure the input UART lines
	 */
	gpio_rx_cts_init();

	/**
	 * Configure UART block
	 */
	uart_init();

	/**
	 * Configure DMA1
	 */
	dma_init();

	/**
	 * Configure the output UART lines
	 */
	gpio_tx_rts_init();

	/**
	 * Enable WCTS interrupt
	 */
	wcts_interrupt_enable(eRemoteTxrequest);

	return;
}

/**
  * @brief  Interface to the application to receive data from UART
  *
  * @note	The API is called twice to receive the packet in two steps
  * 		1) The header
  * 		2) The payload
  * 		The UART driver receives the full packet as soon as the application requests the header
  * 		It is the responsability of the application to make sure both the header and the payload are
  * 		located consecutively in the same buffer
  * 		As the UART driver may receives the payload before the application requests for it, the notification
  * 		for the reception of the payload is delayed until requested by the application
  *
  * @param  pbuffer: The address of the buffer to store the data received. The application shall make sure the payload is received
  * 				  in the same buffer and consecutively to the header
  *
  * @retval None
  */
void LPPUART_receive_data(uint8_t * pbuffer)
{
	if(sUartContext.eHostRxState == eHostHeaderToRx)
	{
		sUartContext.eHostRxState = eHostPayloadToRx;

		/**
		 * Receive the header
		 */
		sUartContext.pRxBuffer = pbuffer;

		__HAL_DMA_DISABLE(&hdma_rx);
		__HAL_LPPUART_DMA_SET_COUNTER(&hdma_rx, PACKET_HEADER_SIZE);
		__HAL_LPPUART_DMA_SET_MEMORY_ADDRESS(&hdma_rx, (uint32_t)pbuffer);
		__HAL_DMA_ENABLE(&hdma_rx);

	}
	else
	{
		sUartContext.eHostRxState = eHostHeaderToRx;

		/**
		 * Receive the payload
		 */
		RxPayloadNotification(eApplication);
	}
    return;
}

/**
  * @brief  Interface to the application to send data over UART
  *
  * @note
  *
  * @param  pbuffer: The address of the buffer holding the data to send
  *
  * @param  uhSize: The size of data to send
  *
  * @retval None
  */
void LPPUART_send_data(uint8_t * pbuffer, uint16_t uhSize)
{
	/**
	 * Enable transmit flag
	 */
	sUartContext.uwTxFlag = ENABLE;

	sUartContext.uwWCTS_TxStarted = ENABLE;

	/**
	 * Disable WCTS interrupt
	 */
	wcts_interrupt_disable();

	sUartContext.uhTxSize = uhSize;

	/**
	 * Enable WCTS HW flow control
	 */
	LPPUART_MSP_ENABLE_UART_HW_CTS(&huart);

	/**
	 * Request High Speed clock
	 */
	LPPUART_HSclkRequest(eLPPUART_Tx, eLPPUART_HSclkEnable);

	/**
	 *  Setup  DMA to be ready to send data
	 */
	sUartContext.pTxBuffer = pbuffer;
	__HAL_DMA_DISABLE(&hdma_tx);
	__HAL_LPPUART_DMA_SET_COUNTER(&hdma_tx, sUartContext.uhTxSize-1);
	__HAL_LPPUART_DMA_SET_MEMORY_ADDRESS(&hdma_tx, (uint32_t)pbuffer);

	/**
	 * Check WCTS GPIO line
	 */
	if(HAL_GPIO_ReadPin(USART_WCTS_GPIO_PORT, USART_WCTS_PIN) == GPIO_PIN_SET)
	{
		/**
		 * Wakeup remote
		 */

		/**
		 * Enable receiving path
		 */
		set_wrts_gpio_mode(GPIO_MODE_AF_PP);

		/**
		 * Wait for ack
		 */
		wcts_interrupt_enable(eAckToTxRequest);
	}
	else
	{
		__HAL_DMA_ENABLE(&hdma_tx);	/**< Enable DMA to start transmission */

	    /**
	     * Enable receiving path
	     */
		set_wrts_gpio_mode(GPIO_MODE_AF_PP);

		/**
		 * remove PU
		 */
		SetGPIO_PUPD(USART_RX_GPIO_PORT, USART_RX_PIN_SOURCE, GPIO_NOPULL);
		SetGPIO_PUPD(USART_WCTS_GPIO_PORT, USART_WCTS_PIN_SOURCE, GPIO_NOPULL);
	}

    return;
}

/**
  * @brief  CTS interrupt handler
  *
  * @note	The Low Power Protocol handles the CTS line as either hardware flow control or low power control signal
  * 		The CTS interrupt handler manages the low power control signaling
  *
  * @param  None
  *
  * @retval None
  */
void LPPUART_wcts_handler(void)
{
	uint8_t		*plocalTxbuffer;
	uint16_t	localTxSize;

	/**
	 * Clear pending bit and mask out interrupt
	 */
	wcts_interrupt_disable();

	switch(sUartContext.eWCTSEventWaited)
	{
		case eLast2ndSentByteAck:
			sUartContext.uwTxFlag = DISABLE;

			/**
			 * Stop flow
			 */
			HAL_GPIO_WritePin(USART_WRTS_GPIO_PORT, USART_WRTS_PIN, GPIO_PIN_SET);
			set_wrts_gpio_mode(GPIO_MODE_OUTPUT_PP);

			LPPUART_MSP_DISABLE_UART_HW_CTS(&huart);

			if(sUartContext.uwRxFlag == DISABLE)
			{
				SetGPIO_PUPD(USART_WCTS_GPIO_PORT, USART_WCTS_PIN_SOURCE, GPIO_PULLUP);
			}
			wcts_interrupt_enable(eLastSentByteAck);
			plocalTxbuffer = sUartContext.pTxBuffer;
			localTxSize =  sUartContext.uhTxSize;
			*(uint8_t*)__HAL_LPPUART_UART_GET_TX_DATA_REGISTER_ADDRESS(&huart) = *(plocalTxbuffer + localTxSize - 1);

			break;

		case eRemoteTxrequest:
			sUartContext.uwRxFlag = ENABLE;

			LPPUART_HSclkRequest(eLPPUART_Rx, eLPPUART_HSclkEnable);

			/**
			 * Enable receiving path
			 */
			set_wrts_gpio_mode(GPIO_MODE_AF_PP);

			SetGPIO_PUPD(USART_WCTS_GPIO_PORT, USART_WCTS_PIN_SOURCE, GPIO_NOPULL);
			SetGPIO_PUPD(USART_RX_GPIO_PORT, USART_RX_PIN_SOURCE, GPIO_NOPULL);
			break;

		case eAckToTxRequest:
			__HAL_DMA_ENABLE(&hdma_tx);	/**< Enable DMA to start transmission */

			SetGPIO_PUPD(USART_WCTS_GPIO_PORT, USART_WCTS_PIN_SOURCE, GPIO_NOPULL);
			SetGPIO_PUPD(USART_RX_GPIO_PORT, USART_RX_PIN_SOURCE, GPIO_NOPULL);
			break;

		case eLastSentByteAck:
			sUartContext.uwWCTS_TxStarted = DISABLE;

			wcts_interrupt_enable(eRemoteTxrequest);
			break;

		default:
			break;
	}

	return;
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
