/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    01-October-2014
  * @brief   Main body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2014 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "app_conf.h"
#include "msp_main.h"
#include "stm32l0xx_nucleo.h"
#include "app_uart_stream.h"
#include "ble_dtm_app.h"
#include "blocking.h"
#include "ble_encode_access.h"

/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
/* Private defines -----------------------------------------------------------*/
#define CONN_CHIP_RESET_TIME		5	/**< The time to keep the reset line to the nRF51822 low ( 2 milliseconds). */
#define CONN_CHIP_STARTUP_TIME		19  /**< Wait for the nRF device to be ready before sending any command ( 7.5 milliseconds). */

#define	DTM_RX_PIN_NUMBER		10
#define	DTM_TX_PIN_NUMBER		11

/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static volatile uint8_t	ubnRFresetTimerLock;
pf_TIMER_TimerCallBack_t pfTimerBLECallBack;
static RTC_HandleTypeDef hrtc;	/**< RTC handle */

/* Private function prototypes -----------------------------------------------*/
static void pf_nRFResetTimerCallBack(void);
static void Init_RTC(void);
static void connectivity_chip_reset(void);
static void dtm_mode_init(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  This function sends a command to the nRF device to enable DTM mode
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void dtm_mode_init(void)
{
    app_uart_stream_comm_params_t uart_params;

    uart_params.baud_rate  = UART_BAUD_RATE_57600;
    uart_params.rx_pin_no  = DTM_RX_PIN_NUMBER;
    uart_params.tx_pin_no  = DTM_TX_PIN_NUMBER;

    ble_dtm_init(&uart_params);

    blocking_resp_wait();

    return;
}



/**
  * @brief  This function notify when then nRF51 nRESET may be released
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void pf_nRFResetTimerCallBack(void)
{
	ubnRFresetTimerLock = 0;

	return;
}

/**
  * @brief  Initialize RTC block
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void Init_RTC(void)
{

	/* Initialize the HW - 37Khz LSI being used*/
	/* Enable the LSI clock */
	__HAL_RCC_LSI_ENABLE();

	/* Enable power module clock */
	__PWR_CLK_ENABLE();

	/* Enable acces to the RTC registers */
	HAL_PWR_EnableBkUpAccess();

	/* Select LSI as RTC Input */
	__HAL_RCC_RTC_CONFIG(RCC_RTCCLKSOURCE_LSI);

	/* Enable RTC */
	__HAL_RCC_RTC_ENABLE();

	hrtc.Instance = RTC;				/**< Define instance */
	hrtc.Lock = HAL_UNLOCKED;			/**< Initialize lock */
	hrtc.State = HAL_RTC_STATE_READY;	/**< Initialize state */

	/* Disable Write Protection */
	__HAL_RTC_WRITEPROTECTION_DISABLE(&hrtc) ;

	/* Wait for LSI to be stable */
	while(__HAL_RCC_GET_FLAG(RCC_FLAG_LSIRDY) == 0);

	return;
}

/**
  * @brief  Function for resetting the nRF51 with IO PC7.
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
static void connectivity_chip_reset(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	uint8_t	ubnRFResetTimerID;

	__GPIOC_CLK_ENABLE();	/**< Switch ON GPIO AHB clock */

	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_7, GPIO_PIN_RESET);	/**< Drive low the GPIO PC7 to reset nRF51 */
  	GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  	GPIO_InitStructure.Pull = GPIO_NOPULL;
  	GPIO_InitStructure.Pin = GPIO_PIN_7;
  	GPIO_InitStructure.Speed = GPIO_SPEED_HIGH;
  	HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);


    TIMER_Create(eTimerModuleID_Interrupt, &ubnRFResetTimerID, eTimerMode_SingleShot, pf_nRFResetTimerCallBack);
    TIMER_Start(ubnRFResetTimerID, CONN_CHIP_RESET_TIME);
    ubnRFresetTimerLock = 1;
    while(ubnRFresetTimerLock == 1);

	/*
	 * Set IO pin in Analog mode
	 */
  	GPIO_InitStructure.Mode = GPIO_MODE_ANALOG;
  	GPIO_InitStructure.Pull = GPIO_NOPULL;
  	GPIO_InitStructure.Pin = GPIO_PIN_7;
  	HAL_GPIO_Init(GPIOC, &GPIO_InitStructure);

  	__GPIOC_CLK_DISABLE();	/**< Switch OFF GPIO AHB clock */

	/*
	 * Wait for the nRF device to be ready before sending any command
	 */
    TIMER_Start(ubnRFResetTimerID, CONN_CHIP_STARTUP_TIME);
    ubnRFresetTimerLock = 1;
    while(ubnRFresetTimerLock == 1);
    TIMER_Delete(ubnRFResetTimerID);


	return;
}

/* Public functions ----------------------------------------------------------*/

/**
  * @brief  UART Low power mode request
  *
  * @note
  *
  * @param  eHSclkRequester: Requester Id
  *
  * @param  eHSclkMode: Mode requested
  *
  * @retval None
  */
void LPPUART_HSclkRequest(eLPPUART_HSclkRequester_t eHSclkRequester, eLPPUART_HSclkMode_t eHSclkMode)
{
	if(eHSclkMode == eLPPUART_HSclkEnable)
	{
		LPM_Mode_Request((eLPM_Id) eHSclkRequester, eLPM_Mode_Sleep);
	}
	else
	{
		LPM_Mode_Request((eLPM_Id) eHSclkRequester, eLPM_Mode_StandBy);
	}

	return;
}


/**
  * @brief  Function for application main entry.
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
int main(void)
{
	/*
	 * The APB power block clock is enable during the startup phase
	 */

#if (JTAG_SUPPORTED == 1)
	/*
	 * Keep debugger enable while in any low power mode
	 */
	__DBGMCU_CLK_ENABLE();
	HAL_DBG_LowPowerConfig(DBGMCU_SLEEP | DBGMCU_STOP | DBGMCU_STANDBY, ENABLE);
#endif

	/**
	 *  HAL library initialization:
	 *    - Configure the Flash prefetch, Flash preread and Buffer caches
	 *    - Systick timer is configured by default as source of time base, but user
	 *          can eventually implement his proper time base source (a general purpose
	 *          timer for example or other time source), keeping in mind that Time base
	 *          duration should be kept 1ms since PPP_TIMEOUT_VALUEs are defined and
	 *          handled in milliseconds basis.
	 *    - Low Level Initialization
	 */
	HAL_Init();

	/*
	 * System clock initialisation
	 */
	SystemClock_Config();

	__HAL_PWR_CLEAR_FLAG(PWR_FLAG_SB);	/**< Clear Standby flag */
	__HAL_RCC_CLEAR_RESET_FLAGS();		/**< clear all reset flags */

	SystemPower_Config();

	/********************************************************************/

	LPM_Mode_Request(eLPM_Application, eLPM_Mode_LP_Stop);	/**< Define minimum low power mode allowed in the system */

	Init_RTC();
    TIMER_Init(&hrtc);

	connectivity_chip_reset();	/**< Reset connectivity chip. */

	UART_BLE_Init();			/**< Request all resources for the UART communication with the nRF device */

    ble_encode_open(); /**< Open encoder module. */

    blocking_init();

	dtm_mode_init();

	/*
	 * Enter main loop.
	 */
    for (;;)
    {
    	Background(SD_COMMAND_ALLOWED);
	}
}

/**
  * @brief  Request action to the scheduler in background
  *
  * @note
  *
  * @param  eMAIN_Backround_Task_Id: Id of the request
  *
  * @retval None
  */
void TaskExecutionRequest(eMAIN_Backround_Task_Id_t eMAIN_Backround_Task_Id)
{
	return;
}

/**
  * @brief  Notify the action in background has been completed
  *
  * @note
  *
  * @param  eMAIN_Backround_Task_Id: Id of the request
  *
  * @retval None
  */
void TaskExecuted(eMAIN_Backround_Task_Id_t eMAIN_Backround_Task_Id)
{
	return;
}

/**
  * @brief  Background task
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
void Background (uint32_t event_mask)
{
	/**
	 * Power management
	 */
	LPM_Enter_Mode();

	return;
}

/**
  * @brief  Actions to execute when going out of stop mode
  *
  * @note
  *
  * @param  None
  *
  * @retval None
  */
USED void LPM_ExitStopMode(void)
{
	 return;
}

/**
  * @brief  Timer notification from the timer server
  *
  * @note	This is the location where the context may be changed to process the timer callback
  *
  * @param  eTimerModuleID: Identifier of the process owner of the callback
  *
  * @param  pfTimerCallBack: Callback to execute
  *
  * @retval None
  */
void TIMER_Notification(eTIMER_ModuleID_t eTimerModuleID, pf_TIMER_TimerCallBack_t pfTimerCallBack)
{
	switch(eTimerModuleID)
	{
		default:
			if(pfTimerCallBack != 0)
			{
				pfTimerCallBack();
			}
			break;
	}

	return;
}

/**
  * @brief  This function handles all transfer over UART to the BLE module
  *
  * @note	This is the location where the context may be changed to process the message
  *
  * @param  pf_PhyDriverEvent_Handler: Callback
  *
  * @param  pPhyDriverEvent:	Type of notification
  *
  * @retval None
  */
void LPPUART_Msg_Handler(pf_LPPUART_PhyDriverEvent_Handler_t pf_PhyDriverEvent_Handler, sLPPUART_PhyDriverEvent_t pPhyDriverEvent)
{
	pf_PhyDriverEvent_Handler(pPhyDriverEvent);

	return;
}

/**
  * @brief  This function is empty to avoid starting the SysTick Timer
  * @param  None
  * @retval None
  */
HAL_StatusTypeDef HAL_InitTick(uint32_t TickPriority)
{
  return (HAL_OK);
}

/**@brief Function for error handling, which is called when an error has occurred.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 * @retval None
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    while(1);
}



/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
